package de.opalpiraten.scenepainter.gui;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.beans.*;
import java.net.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * Edits scene properties
 */
public class SceneEditor extends JInternalFrame implements ActionListener,
ListSelectionListener, ChangeListener, SceneListener,
WorkspaceListener {

  /**
   * Horizontal gridsize
   */
  private JSpinner gridw = new JSpinner(new SpinnerNumberModel(10,1,100, 1));
  
  /**
   * Vertical gridsize
   */
  private JSpinner gridh = new JSpinner(new SpinnerNumberModel(10,1,100, 1));
  
  /**
   * Enable/Disable gridsnapping
   */
  private JCheckBox snap = new JCheckBox(Dict.get("gui.sceneeditor.snap"));
  
  /**
   * Lists the layers in the Scene
   */
  private JList layers = new JList();
  
  /**
   * User wants to raise a layer
   */
  private JButton raise = new JButton(Util.createIcon("toolbarButtonGraphics/navigation/Up16.gif",""));
  
  /**
   * User wants to lower a layer
   */
  private JButton lower = new JButton(Util.createIcon("toolbarButtonGraphics/navigation/Down16.gif",""));
  
  /**
   * User wants to move a layer horizontically
   * NOTE: The limits are for layout purposes only.
   */
  private JSpinner movex = new JSpinner(new SpinnerNumberModel(0,-9999,9999, 1));
  
  /**
   * User wants to move a layer vertically
   * NOTE: The limits are for layout purposes only.
   */
  private JSpinner movey = new JSpinner(new SpinnerNumberModel(0,-9999,9999, 1));
  
  
  /**
   * User wants to delete a layer
   */
  private JButton delete = new JButton(Util.createIcon("toolbarButtonGraphics/general/Delete16.gif",""));
  
  /**
   * User wants a a bounding box drawn around the selected element
   */
  private JCheckBox bbox = new JCheckBox(Dict.get("gui.sceneeditor.bbox"));
  
  /**
   * User wants to set the grid policy
   */
  private JCheckBox ontop = new JCheckBox(Dict.get("gui.sceneeditor.ontop"));
  
  /**
   * The alpha level of the layer
   */
  private JSpinner alpha = new JSpinner(new SpinnerNumberModel(100, 0, 100, 10));
  
  /**
   * Occasionally a different color is needed for the grid
   */
  private JTextField gridcol = new JTextField(6);
  
  /**
   * Occasionally a different color is needed for the background of the scene
   */
  private JTextField sceneback = new JTextField(6);
  
  /**
   * Reference to the scene being edited
   */
  private Scene scene;
  
  /**
   * For giving a hint, whether or not the current scene has the dirty flag
   */
  private JPanel dirty = new JPanel();
  
  /**
   * Stop actionlisteners from being executed. This is required when setting
   * the values of widgets without them thinking the update come from the
   * user and trying to update the scene
   */
  private boolean noupdate=false;

  public SceneEditor() {
    super(Dict.get("gui.sceneeditor.title"),false,false,false,true);
    snap.addActionListener(this);
    raise.addActionListener(this);
    lower.addActionListener(this);
    delete.addActionListener(this);
    ontop.addActionListener(this);
    gridcol.addActionListener(this);
    sceneback.addActionListener(this);
    layers.addListSelectionListener(this);
    bbox.addActionListener(this);
    gridh.addChangeListener(this);
    gridw.addChangeListener(this);
    movex.addChangeListener(this);
    movey.addChangeListener(this);
    alpha.addChangeListener(this);
    
    JPanel content = new JPanel();
    
    setContentPane(content);
    content.setLayout(new BoxLayout(content,BoxLayout.Y_AXIS));
    
    JPanel layerctrl = new JPanel();
    layerctrl.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.sceneeditor.layerctrl")));
    GridBagLayout gbl = new GridBagLayout();
    layerctrl.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();


    gbc.anchor=GridBagConstraints.WEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel moves = new JPanel();
    moves.setLayout(new BoxLayout(moves,BoxLayout.X_AXIS));
    moves.add(movex);
    //moves.add(Box.createHorizontalStrut(3));
    moves.add(new JLabel(" : "));
    moves.add(movey);
    gbl.setConstraints(moves,gbc);
    layerctrl.add(moves);
    
    gbc.anchor=GridBagConstraints.EAST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=1;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(alpha,gbc);
    layerctrl.add(alpha);
    
    gbc.anchor=GridBagConstraints.NORTHEAST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=2;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(dirty,gbc);
    layerctrl.add(dirty);
    
    gbc.anchor=GridBagConstraints.EAST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=2; 
    gbc.gridheight=3;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    JScrollPane jsp0 = new JScrollPane(layers);
    gbl.setConstraints(jsp0,gbc);
    layerctrl.add(jsp0);
    

    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=2;
    gbc.gridy=1;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(raise,gbc);
    layerctrl.add(raise);

    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=2;
    gbc.gridy=2;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(lower,gbc);
    layerctrl.add(lower);

    gbc.anchor=GridBagConstraints.SOUTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=2;
    gbc.gridy=3;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(delete,gbc);
    layerctrl.add(delete);


    
    content.add(layerctrl);

    JPanel gridctrl = new JPanel();
    
    gridctrl.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.sceneeditor.gridctrl")));
    gridctrl.setLayout(new BoxLayout(gridctrl,BoxLayout.Y_AXIS));
    
    JPanel subgridctrl0 = new JPanel();
    subgridctrl0.add(gridw);
    subgridctrl0.add(new JLabel("x"));
    subgridctrl0.add(gridh);
    subgridctrl0.add(snap);

    JPanel subgridctrl1 = new JPanel();
    subgridctrl1.setLayout(new BoxLayout(subgridctrl1,BoxLayout.X_AXIS));
    subgridctrl1.add(ontop);
    subgridctrl1.add(bbox);
    
    JPanel subgridctrl2 = new JPanel();
    subgridctrl2.add(new JLabel(Dict.get("gui.sceneeditor.gridctrl.color.grid")));
    subgridctrl2.add(gridcol);
    subgridctrl2.add(new JLabel(Dict.get("gui.sceneeditor.gridctrl.color.background")));
    subgridctrl2.add(sceneback);
    
    gridctrl.add(subgridctrl0);
    gridctrl.add(subgridctrl1);
    gridctrl.add(subgridctrl2);
    content.add(gridctrl);
    
    bbox.setSelected(true);

    pack();
  }
  
  public void replaceProject(Project p) {}
  
  public void replaceScene(Scene s) {
    if (scene!=null) scene.removeSceneListener(this);
    s.addSceneListener(this);
    if (s.dirty) dirty.setBackground(Color.RED.darker());
    else dirty.setBackground(Color.GREEN.darker());
    sceneChanged(s);
  }
  
  // Interface implemented
  public void sceneChanged(Scene s) {
    noupdate=true; // Disable widgets listeners
    scene=s;
    if (s.dirty) dirty.setBackground(Color.RED.darker());
    else  dirty.setBackground(Color.GREEN.darker());
    
    SceneElement sel =s.getSelected();
    snap.setSelected(s.isSnapgrid());
    layers.removeListSelectionListener(this);
    
    // Need to copy it. Otherwise every repaint of the dialog triggers
    // a toString() of all elements, which is expensive (with increasing
    // number of elements). This is especially bad, if this editor 
    // overlaps a scene (repainting the scene then retriggers repainting
    // the editor).
    SceneElement[] elems = s.getElements();
    String[] names = new String[elems.length];
    for (int i=0;i<names.length;i++) names[i]=elems[i].toString();
    layers.setListData(names);
    
    layers.setSelectedIndex(s.getSelectedIndex());
    layers.addListSelectionListener(this);
    Grid g = scene.getGrid();
    
    if (g!=null) {
      Dimension d = g.getCellsize();
      gridw.setValue(new Integer(d.width));
      gridh.setValue(new Integer(d.height));
      gridcol.setText(Util.toHexString(g.getLinecolor()));
    }
    ontop.setSelected(scene.isOntopgrid());
    if (sel!=null) {
      Point p = s.getSelected().getPosition();
      movex.setValue(new Integer(p.x));
      movey.setValue(new Integer(p.y));
      alpha.setValue(new Integer((int)(sel.getAlpha()*100)));
      layers.ensureIndexIsVisible(scene.getSelectedIndex());
    }
    if (bbox.isSelected()) scene.setSelectedColor(Color.BLUE);
    
    sceneback.setText(Util.toHexString(scene.getBackground()));
    noupdate=false;
  }
  
  public void actionPerformed(ActionEvent e) {
    if (noupdate) return;
    Object src = e.getSource();

    if (src==snap) {
      scene.setSnapgrid(snap.isSelected());
    }
    
    if (src==delete) {
      int i=layers.getSelectedIndex();
      if (i<0) return;
      scene.remove(scene.getElements()[i]);
      scene.setSelectedIndex(0);
    }
    
    if (src==raise) {
      int i=layers.getSelectedIndex();
      scene.setSelectedIndex(i);
      scene.raise();
    }
    
    if (src==lower) {
      int i=layers.getSelectedIndex();
      scene.setSelectedIndex(i);
      scene.lower();
    }
    
    if (src==bbox) {
      if (bbox.isSelected()) scene.setSelectedColor(Color.BLUE);
      else scene.setSelectedColor(null);
    }
    
    if (src==ontop) {
      scene.setOntopgrid(ontop.isSelected());
    }
    
    if (src==gridcol) {
      Grid g = scene.getGrid();
      try {
        g.setLinecolor(Color.decode(gridcol.getText()));
        scene.setGrid(g);
      }
      catch (Exception exp) {
      }
    }
    
    if (src==sceneback) {
      Grid g = scene.getGrid();
      try {
        scene.setBackground(Color.decode(sceneback.getText()));
        scene.setGrid(g);
      }
      catch (Exception exp) {
      }
    }
  }
  
  public void stateChanged(ChangeEvent e) {
    if (noupdate) return;
    Object src = e.getSource();
    
    if (src==gridw || src==gridh) {
      try {
        int w = 10;
        int h = 10;
        w = ((Integer)gridw.getValue()).intValue();
        h = ((Integer)gridh.getValue()).intValue();
        if (w<1) w=1;
        if (h<1) h=1;
        Grid g = new Grid();
        g.setCellsize(new Dimension(w,h));
        scene.setGrid(g);
      }
      catch (Exception exp) {}
    }
    
    if (src==movex || src==movey) {
      try {
        int x=0;
        int y=0;
         x = ((Integer)movex.getValue()).intValue();
         y = ((Integer)movey.getValue()).intValue();
         scene.move(scene.getSelected(),new Point(x,y));
      }
      catch(Exception exp) {}         
    }
    
    if (src==alpha) {
      float i =((Integer)alpha.getValue()).intValue();
      int l  = scene.getSelectedIndex();
      scene.setAlpha(l,i/100 );
    }
  }
  
  public void valueChanged(ListSelectionEvent e) {
    if (scene==null || e.getValueIsAdjusting() || noupdate ) return;
    int i=layers.getSelectedIndex();
    scene.setSelectedIndex(i);
  }
  
}